/*
 * playerlist.c
 */
#include "copyright.h"

#include <stdio.h>
#include "Wlib.h"
#include "defs.h"
#include "struct.h"
#include "data.h"

#ifdef GALAXY
static char    *classes[NUM_TYPES] =
{
   "SC", "DD", "CA", "BB", "AS", "SB", "GA", "  "
};
#else
static char    *classes[NUM_TYPES] =
{
   "SC", "DD", "CA", "BB", "AS", "SB", "  "
};
#endif				/* GALAXY */

playerlist()
{
   int             i;
   char            buf[100];

   /*
    * W_ClearWindow(playerw);
    */
#ifdef NEW_PL
   if(new_playerlist)
      (void) strcpy(buf, "  Type Rank      Name               Kills      Login            Host");
   else
#endif
      (void) strcpy(buf, "  Type Rank      Name            Kills   Win  Loss  Ratio Offense Defense     DI");
   W_WriteText(playerw, 0, 1, textColor, buf, strlen(buf), W_RegularFont);
   for (i = 0; i < MAXPLAYER; i++) {
      updatePlayer[i] = 1;
   }
   playerlist2();
}

#ifdef EM
static int      PlayerPos[MAXPLAYER];
/* player list sorting inspired by jnm@crown.berkeley.EDU */
Sorted_playerlist2()
{
   register int    i, h, pos = 1;
   char            buf[100];
   register struct player *j;
   int             kills, losses;
   double          ratio;
   float           pRating, oRating, dRating, bRating;

   for (h = -1; h < MAXTEAM; h++) {
      for (i = 0, j = &players[i]; i < MAXPLAYER; i++, j++) {
	 if (j->p_team != ((h < 0) ? NOBODY : (1 << h)))
	    continue;
	 /* mask wrong teams */

	 /* make sure the player is actually in the game! */
	 if (!(j->p_status == PALIVE || j->p_status == PEXPLODE
	       || j->p_status == PDEAD ||
	       j->p_status == POUTFIT))
	    continue;
	 /* this player should have an entry on the list */
	 pos++;

	 /* only update them if updatePlayer or position has moved */
	 if (!updatePlayer[i] && PlayerPos[pos] == i)
	    continue;

	 PlayerPos[pos] = i;
	 updatePlayer[i] = 0;
	 if (j->p_status != PALIVE) {
	    /* XFIX */
	    W_ClearArea(playerw, 0, pos, 83, 1);
	    continue;
	 }
#ifdef NEW_PL
	 if(new_playerlist){
	    int	l;
	    l = build_newplayerbuf(buf, j);
	    W_WriteText(playerw, 0, pos, playerColor(j), buf, l,
			shipFont(j));
	    continue;
	 }
#endif
	 if (j->p_ship.s_type == STARBASE) {
	    kills = j->p_stats.st_sbkills;
	    losses = j->p_stats.st_sblosses;
	 } else {
	    kills = j->p_stats.st_kills + j->p_stats.st_tkills;
	    losses = j->p_stats.st_losses + j->p_stats.st_tlosses;
	 }
	 if (losses == 0) {
	    ratio = kills;
	 } else {
	    ratio = (double) kills / losses;
	 }
	 if (!j->p_stats.st_tticks) {
	    oRating = 0.0;
	    pRating = 0.0;
	    bRating = 0.0;
	 } else {
	    oRating = offenseRating(j);
	    pRating = planetRating(j);
	    bRating = bombingRating(j);
	 }
	 dRating = defenseRating(j);
	 (void) sprintf(buf, "%c%c %s  %-9.9s %-16.16s%5.2f %5d %5d %6.2lf   %5.2f   %5.2f %8.2f",
			teamlet[j->p_team],
			shipnos[j->p_no],
			classes[j->p_ship.s_type],
			ranks[j->p_stats.st_rank].name,
			j->p_name,
			j->p_kills,
			kills,
			losses,
			ratio,
			oRating,
			dRating,
	  (oRating + pRating + bRating) * (j->p_stats.st_tticks / 36000.0));
	 W_WriteText(playerw, 0, pos, playerColor(j), buf, strlen(buf),
		     shipFont(j));
      }
   }

   /* clear out all unused areas */
   for (pos++; pos < MAXPLAYER; pos++) {
      if (PlayerPos[pos] != -1) {
	 PlayerPos[pos] = -1;
	 W_ClearArea(playerw, 0, pos, 83, 1);
      }
   }
}
#endif


playerlist2()
{
   register int    i;
   char            buf[100], *bufc = buf;
   register struct player *j;
   int             kills, losses;
   double          ratio;
   float           pRating, oRating, dRating, bRating;

   if (!W_IsMapped(playerw))
      return;

#ifdef EM
   if (sortPlayers) {
      Sorted_playerlist2();
      return;
   }
#endif
   for (i = 0, j = &players[i]; i < MAXPLAYER; i++, j++) {
#ifdef EM
      PlayerPos[i] = -2;	/* force a clear next time player sorting is
				 * on */
#endif
      if (!updatePlayer[i])
	 continue;
      updatePlayer[i] = 0;
      if (j->p_status != PALIVE) {
	 /* XFIX */
	 W_ClearArea(playerw, 0, i + 2, 83, 1);
	 continue;
      }
#ifdef NEW_PL
      if(new_playerlist){
	 int	l;
	 l = build_newplayerbuf(buf, j);
	 W_WriteText(playerw, 0, i + 2, playerColor(j), buf, l,
		     shipFont(j));
	 continue;
      }
#endif
	 
      if (j->p_ship.s_type == STARBASE) {
	 kills = j->p_stats.st_sbkills;
	 losses = j->p_stats.st_sblosses;
      } else {
	 kills = j->p_stats.st_kills + j->p_stats.st_tkills;
	 losses = j->p_stats.st_losses + j->p_stats.st_tlosses;
      }
      if (losses == 0) {
	 ratio = kills;
      } else {
	 ratio = (double) kills / losses;
      }
      if (!j->p_stats.st_tticks) {
	 oRating = 0.0;
	 pRating = 0.0;
	 bRating = 0.0;
      } else {
	 oRating = offenseRating(j);
	 pRating = planetRating(j);
	 bRating = bombingRating(j);
      }
      dRating = defenseRating(j);
      (void) sprintf(buf, "%c%c %s  %-9.9s %-16.16s%5.2f %5d %5d %6.2lf   %5.2f   %5.2f %8.2f",
		     teamlet[j->p_team],
		     shipnos[j->p_no],
		     classes[j->p_ship.s_type],
		     ranks[j->p_stats.st_rank].name,
		     j->p_name,
		     j->p_kills,
		     kills,
		     losses,
		     ratio,
		     oRating,
		     dRating,
	  (oRating + pRating + bRating) * (j->p_stats.st_tticks / 36000.0));
      W_WriteText(playerw, 0, i + 2, playerColor(j), buf, strlen(buf),
		  shipFont(j));
   }
}
#ifdef NEW_PL

int
build_newplayerbuf(buf, j)

   char			*buf;
   struct player	*j;
{
   extern char		*itof42();
   register char	*s = buf, *r;
   register		i;

   *s++ = teamlet[j->p_team];
   *s++ = shipnos[j->p_no];
   *s++ = ' ';
   strcpy(s, classes[j->p_ship.s_type]);
   s+= 2;
   *s++ = ' ';
   *s++ = ' ';
   for(i=0, r=ranks[j->p_stats.st_rank].name; i< 9; i++){
      if(*r)
	 *s++ = *r++;
      else
	 *s++ = ' ';
   }
   *s++ = ' ';
   for(i=0,r=j->p_name; i < 16; i++){
      if(*r)
	 *s++ = *r++;
      else
	 *s++ = ' ';
   }
   *s++ = ' ';
   s = itof42(s, j->p_kills);
   *s++ = ' ';
   *s++ = ' ';
   *s++ = ' ';
   *s++ = ' ';
   *s++ = ' ';
   *s++ = ' ';
   /* login@host */
   for(i=0,r=j->p_login; i < 16; i++){
      if(*r)
	 *s++ = *r++;
      else
	 *s++ = ' ';
   }
   *s++ = ' ';
   for(i=0,r=j->p_monitor; i < 16; i++){
      if(*r)
	 *s++ = *r++;
      else
	 break;
   }
   return s - buf;
}

#endif /* NEW_PL */
